.CODE 

InitGDIp PROC USES ebx hWnd:DWORD

;--------------------------------------------------------------------
; InitGDIp - create objects we need to draw with
; IN:   control [window] handle
; OUT:  NULL - no error return code until now
; TODO: Vertical orientation brush, error handling
;--------------------------------------------------------------------

Local rct:RECT
local rctF:RectF
Local hBrush1:DWORD
Local hBrush2:DWORD
Local dwGradient:DWORD

;-- Get pointer to CpuPgBar struct
	Invoke GetWindowLong,hWnd,0
	mov ebx,eax
    ASSUME ebx:PTR CpuPgBar
    
;-- Save control style
	Invoke GetWindowLong,hWnd,GWL_STYLE
	mov [ebx].dwStyle,eax
	
	invoke ClpgInitStandard,hWnd

;-- INITIALIZE GRAPHICS CLASS
	Invoke GdipCreateFromHWND,hWnd,ADDR [ebx].hGraphics
;-- Draw text antialiased
	Invoke GdipSetTextRenderingHint,[ebx].hLGraphics,TextRenderingHintAntiAlias
;-- create background brush
	Invoke GdipCreateSolidFill, [ebx].dwColorBckgr,ADDR [ebx].hBrush	
	Invoke GetClientRect,hWnd,ADDR rct
	
;-- create back buffer bitmap
	Invoke GdipCreateBitmapFromGraphics,rct.right,rct.bottom,[ebx].hGraphics,ADDR [ebx].hBitmap
;-- create graphics context for back buffer bitmap 
	Invoke GdipGetImageGraphicsContext,[ebx].hBitmap,ADDR [ebx].hLGraphics
;	Invoke GdipSetSmoothingMode,[ebx].hLGraphics,SmoothingModeAntiAlias

;-- CREATE TEXTURE BRUSH
	mov eax,[ebx].dwStyle
	and eax,CLPG_ORIENTATIONVERTICAL
	.If ! eax    ; Orientation horizontal
	;-- create gradient brush from 0% to 50% 
		mov ecx,rct.right
		push ecx
		shr ecx,1
		add ecx,1
		mov rct.right,ecx
		Invoke GdipCreateLineBrushFromRectI,ADDR rct,[ebx].dwColor000,[ebx].dwColor050,0,0,ADDR hBrush1
;		Invoke GdipSetLineBlend,hBrush1,focus,scale
	;-- Draw brush to back buffer bitmap
		Invoke GdipFillRectangleI,[ebx].hLGraphics,hBrush1,rct.left,0,rct.right,rct.bottom
		Invoke GdipDeleteBrush,hBrush1
		
	;-- create gradient brush from 50% to 100%	
		pop ecx
		shr ecx,1
		add ecx,1
		mov rct.right,ecx
		sub ecx,2
		mov rct.left,ecx
		Invoke GdipCreateLineBrushFromRectI,ADDR rct,[ebx].dwColor050,[ebx].dwColor100,0,0,ADDR hBrush2
		add rct.left,1
	;-- Draw brush to back buffer bitmap
		Invoke GdipFillRectangleI,[ebx].hLGraphics,hBrush2,rct.left,0,rct.right,rct.bottom
		Invoke GdipDeleteBrush,hBrush2
		
	;-- Draw vertical grid lines if required
		mov eax,[ebx].dwStyle
		and eax,CLPG_HATCHSTYLEVERTICAL
		.If eax
			Invoke GetClientRect,hWnd,ADDR rct
			Invoke GdipCreateHatchBrush,HatchStyleVertical,55555555h,0,ADDR hBrush1
			Invoke GdipFillRectangleI,[ebx].hLGraphics,hBrush1,0,0,rct.right,rct.bottom
			Invoke GdipDeleteBrush,hBrush1
		.endif
		
	.Else        ; ORIENTATION VERTICAL
	;-- create gradient brush from 0% to 50% 
		mov ecx,rct.bottom
		shr ecx,1
		sub ecx,1
		mov rct.top,ecx
		add ecx,2
		mov rct.bottom,ecx
		Invoke GdipCreateLineBrushFromRectI,ADDR rct,[ebx].dwColor050,[ebx].dwColor000,1,0,ADDR hBrush1
	;-- Draw brush to back buffer bitmap
		Invoke GdipFillRectangleI,[ebx].hLGraphics,hBrush1,rct.left,rct.top,rct.right,rct.bottom
		Invoke GdipDeleteBrush,hBrush1
		
	;-- create gradient brush from 50% to 100%	
		mov rct.top,0
		Invoke GdipCreateLineBrushFromRectI,ADDR rct,[ebx].dwColor100,[ebx].dwColor050,1,0,ADDR hBrush2
		add rct.left,1
	;-- Draw brush to back buffer bitmap
		Invoke GdipFillRectangleI,[ebx].hLGraphics,hBrush2,0,0,rct.right,rct.bottom
		Invoke GdipDeleteBrush,hBrush2
		
	;-- Draw horizontal grid lines if required
		mov eax,[ebx].dwStyle
		and eax,CLPG_HATCHSTYLEVERTICAL
		.If eax
			Invoke GetClientRect,hWnd,ADDR rct
			Invoke GdipCreateHatchBrush,HatchStyleHorizontal,55555555h,0,ADDR hBrush1
			Invoke GdipFillRectangleI,[ebx].hLGraphics,hBrush1,0,0,rct.right,rct.bottom
			Invoke GdipDeleteBrush,hBrush1
		.endif	
	.Endif

;-- Create brush from bitmap we have drawn
	Invoke GdipCreateTexture,[ebx].hBitmap,0,ADDR [ebx].htBrush

;-- create frame pen
	Invoke GdipCreatePen1,ColorsDarkGray,fWidth,UnitPixel,ADDR [ebx].hPen
;	Invoke GdipSetPenLineJoin,[ebx].hPen,LineJoinRound
	
;-- Initialize CPU Load mode if required
	mov eax,[ebx].dwStyle
	and eax,CLPG_CPULOAD
	.If eax
		Invoke _InitPdhCounter,[ebx].hWndCtrl
	.endif

    ASSUME ebx:NOTHING
	xor eax,eax
	RET
	
InitGDIp EndP

InitFont PROC USES ebx hWnd:DWORD,lpszFontName:DWORD

;--------------------------------------------------------------------
; InitFont - create font object and brush for text output
; IN:   hWnd: control [window] handle
;       lpszFontName: Pointer to ANSI string (font name)
;                     NULL for font of parent window
; OUT:  NULL - no error return code until now
;--------------------------------------------------------------------

Local logF:LOGFONT
Local rct:RECT

;-- Get pointer to CpuPgBar struct
	Invoke GetWindowLong,hWnd,0
	mov ebx,eax
    ASSUME ebx:PTR CpuPgBar
    
    Invoke GetClientRect,hWnd,ADDR rct
    
;-- Get font name and/or map to Unicode string
	.IF lpszFontName == 0
	;-- Get parent windows fontname
		Invoke GetParent,hWnd
		mov edx,eax
		Invoke SendMessage,edx,WM_GETFONT,0,0
		mov edx,eax
		Invoke GetObject,edx,SIZEOF logF,ADDR logF
		Invoke MultiByteToWideChar,CP_ACP,0,ADDR logF.lfFaceName,32,ADDR [ebx].szwFontName,32
	.ELSE
	;-- user font
		Invoke MultiByteToWideChar,CP_ACP,0,lpszFontName,32,ADDR [ebx].szwFontName,32
	.ENDIF

;-- Create FontFamily
	Invoke GdipCreateFontFamilyFromName,ADDR [ebx].szwFontName,0,ADDR [ebx].hFontFam
	;-- Check if we have a valid FontFamily, if not, create font ARIAL
	;   this is also needed to show text in design mode :)
	.if eax != GpStatusOk
		Invoke GdipCreateFontFamilyFromName,ADDR szwFont,0,ADDR [ebx].hFontFam ; Design mode: Arial 
	.endif

;-- Calculate font size
	finit
	push 100
	mov eax,[ebx].dwStyle
	and eax,CLPG_ORIENTATIONVERTICAL
	.If ! eax    ; Orientation horizontal	
		fild DWORD PTR rct.bottom
	.else
		fild DWORD PTR rct.right
	.endif
	fidiv DWORD PTR [esp]
	fimul DWORD PTR [ebx].dwFontSize
	fstp REAL4 PTR [esp]
	pop edx
	
;-- Create font in requested size
	Invoke GdipCreateFont,[ebx].hFontFam,edx,FontStyleBold,UnitPixel,ADDR [ebx].hCurFont
	
;-- create brush for text output
	Invoke GdipCreateSolidFill,[ebx].dwColorText,ADDR [ebx].hStringBrush
	
	ASSUME ebx:NOTHING
	Ret
InitFont EndP

CleanUpGDIp Proc USES ebx hWnd:DWORD

;--------------------------------------------------------------------
; CleanUpGDIp - destroy gdip classes
; IN:   control [window] handle
; OUT:  NULL - no error return code needed
;--------------------------------------------------------------------

;-- Get pointer to CpuPgBar struct
	Invoke GetWindowLong,hWnd,0
	mov ebx,eax
    ASSUME ebx:PTR CpuPgBar
    
;-- Clean up CPU load style if required 
	mov eax,[ebx].dwStyle
	and eax,CLPG_CPULOAD
	.If eax
		Invoke _CleanupPdhCounter,[ebx].hWndCtrl
	.endif
	
;-- Clean up gdi+ classes
	Invoke GdipDeletePen, [ebx].hPen
	mov [ebx].hPen,0
	Invoke GdipDeleteFont,[ebx].hCurFont
	mov [ebx].hCurFont,0
	Invoke GdipDeleteFontFamily,[ebx].hFontFam
	mov [ebx].hFontFam,0
	Invoke GdipDeleteBrush,[ebx].hBrush
	mov [ebx].hBrush,0
	Invoke GdipDeleteBrush,[ebx].htBrush
	mov [ebx].htBrush,0
	Invoke GdipDeleteBrush,[ebx].hStringBrush
	mov [ebx].hStringBrush,0
	Invoke GdipDisposeImage,[ebx].hBitmap
	mov [ebx].hBitmap,0
	Invoke GdipDeleteGraphics,[ebx].hLGraphics
	mov [ebx].hLGraphics,0
	Invoke GdipDeleteGraphics,[ebx].hGraphics
	mov [ebx].hGraphics,0
	
	ASSUME ebx:NOTHING
	xor eax,eax
	RET
CleanUpGDIp EndP

DrawGDIp Proc USES ebx hWnd:DWORD

;--------------------------------------------------------------------
; DrawGDIp - Control drawing procedure
; IN:   control [window] handle
; OUT:  NULL - no error return code until now
;--------------------------------------------------------------------

local rct:RECT
local rctF:RectF
local rctFL:RectF
local hBitmap:DWORD
local hL2Graphics:DWORD

;-- Get pointer to CpuPgBar struct
	Invoke GetWindowLong,hWnd,0
	mov ebx,eax
	ASSUME ebx:PTR CpuPgBar

	Invoke GetClientRect,hWnd,ADDR rct
	Invoke GdipGetVisibleClipBounds,[ebx].hLGraphics,ADDR rctF
	
;-- Draw background
	Invoke GdipFillRectangleI,[ebx].hLGraphics,[ebx].hBrush,0,0,rct.right,rct.bottom
;-- check for control orientation
	mov eax,[ebx].dwStyle
	and eax,CLPG_ORIENTATIONVERTICAL
	.If ! eax
	;-- Calculate pixel position using FPU
		finit
		push [ebx].dwRange
		fild DWORD PTR rct.right
		fidiv DWORD PTR [esp]
		fimul [ebx].dwRelPos
		fistp DWORD PTR [esp]
		pop edx			; edx holds width in pixels to draw
	;-- Draw ProgBar
	
		mov eax,[ebx].dwStyle
		and eax,CLPG_PARTIAL
		.If ! eax
			Invoke GdipFillRectangleI,[ebx].hLGraphics,[ebx].htBrush,0,0,edx,rct.bottom
		.else
			mov ecx,edx
			sub ecx,15
			mov edx,15
			Invoke GdipFillRectangleI,[ebx].hLGraphics,[ebx].htBrush,ecx,0,edx,rct.bottom
		.endif
	;-- Draw border if flat style
		Invoke GetWindowLong,hWnd,GWL_EXSTYLE
		test eax,WS_EX_CLIENTEDGE
		jnz @F
			sub rct.bottom,1
			sub rct.right,1
			invoke GdipSetPenColor,[ebx].hPen,ColorsBlack
			Invoke GdipDrawRectangleI,[ebx].hLGraphics,[ebx].hPen,0,0,rct.right,rct.bottom
			invoke GdipSetPenColor,[ebx].hPen,ColorsWhite
			add rct.left,1
			add rct.top,1
			sub rct.bottom,2
			sub rct.right,2		
			Invoke GdipDrawRectangleI,[ebx].hLGraphics,[ebx].hPen,rct.left,rct.top,rct.right,rct.bottom
		@@:
	;-- Check if we need to draw text
		mov eax,[ebx].dwStyle
		and eax,CLPG_NOTEXT
		.If ! eax
			Invoke GdipMeasureString,[ebx].hLGraphics,ADDR [ebx].szWBuffer,[ebx].dwStrLen,[ebx].hCurFont,ADDR rctF,0,ADDR rctFL,0,0
			finit
			push 2
			fld REAL4 PTR rctF.Right
			fsub REAL4 PTR rctFL.Right
			fidiv DWORD PTR [esp]
			fstp REAL4 PTR rctF.Left
			
			fld REAL4 PTR rctF.Bottom
			fsub REAL4 PTR rctFL.Bottom
			fidiv DWORD PTR [esp]
			fstp REAL4 PTR rctF.Top
			pop eax
		;-- draw string
			Invoke GdipDrawString,[ebx].hLGraphics,ADDR [ebx].szWBuffer,[ebx].dwStrLen,[ebx].hCurFont,ADDR rctF,0,[ebx].hStringBrush
		.Endif
		
	;-- WE ARE READY, DRAW IMAGE
		Invoke GdipDrawImage,[ebx].hGraphics,[ebx].hBitmap,0,0
		
	.ELSE	; draw vertical
	;-- Calculate pixel position using FPU
		finit
		push [ebx].dwRange
		fild DWORD PTR rct.bottom
		fidiv DWORD PTR [esp]
		fimul [ebx].dwRelPos
		fistp DWORD PTR [esp]
		pop edx	
		mov ecx,rct.bottom
		sub ecx,edx		; ecx holds height in pixels to draw
	;-- Draw ProgBar
		Invoke GdipFillRectangleI,[ebx].hLGraphics,[ebx].htBrush,0,ecx,rct.right,edx
	;-- Draw border if flat style
		Invoke GetWindowLong,hWnd,GWL_EXSTYLE
		test eax,WS_EX_CLIENTEDGE
		jnz @F
			sub rct.bottom,1
			sub rct.right,1
			invoke GdipSetPenColor,[ebx].hPen,ColorsBlack
			Invoke GdipDrawRectangleI,[ebx].hLGraphics,[ebx].hPen,0,0,rct.right,rct.bottom
			invoke GdipSetPenColor,[ebx].hPen,ColorsWhite
			add rct.left,1
			add rct.top,1
			sub rct.bottom,2
			sub rct.right,2		
			Invoke GdipDrawRectangleI,[ebx].hLGraphics,[ebx].hPen,rct.left,rct.top,rct.right,rct.bottom
		@@:
	;-- Check if we need to draw text
		mov eax,[ebx].dwStyle
		and eax,CLPG_NOTEXT
		.If ! eax
			;-- create a bitmap to draw rotated text
			Invoke GetClientRect,hWnd,ADDR rct
			Invoke GdipCreateBitmapFromGraphics,rct.right,rct.bottom,[ebx].hLGraphics,ADDR hBitmap
			invoke GdipImageRotateFlip,hBitmap,Rotate90FlipNone
			Invoke GdipGetImageGraphicsContext,hBitmap,ADDR hL2Graphics
			Invoke GdipGetVisibleClipBounds,hL2Graphics,ADDR rctF
			
			Invoke GdipMeasureString,hL2Graphics,ADDR [ebx].szWBuffer,[ebx].dwStrLen,[ebx].hCurFont,ADDR rctF,0,ADDR rctFL,0,0
			finit
			push 2
			fld REAL4 PTR rctF.Right
			fsub REAL4 PTR rctFL.Right
			fidiv DWORD PTR [esp]
			fstp REAL4 PTR rctF.Left
			
			fld REAL4 PTR rctF.Bottom
			fsub REAL4 PTR rctFL.Bottom
			fidiv DWORD PTR [esp]
			fstp REAL4 PTR rctF.Top
			pop eax
		;-- draw string
			Invoke GdipDrawString,hL2Graphics,ADDR [ebx].szWBuffer,[ebx].dwStrLen,[ebx].hCurFont,ADDR rctF,0,[ebx].hStringBrush		
	
			Invoke GdipImageRotateFlip,hBitmap,Rotate270FlipNone
			invoke GdipDrawImage,[ebx].hLGraphics,hBitmap,0,0
			Invoke GdipDeleteGraphics,hL2Graphics
			Invoke GdipDisposeImage,hBitmap
		.Endif
	;-- Draw to screen
		Invoke GdipDrawImage,[ebx].hGraphics,[ebx].hBitmap,0,0
	.ENDIF

	xor eax,eax
	ASSUME ebx:NOTHING
	RET
	
DrawGDIp EndP

ClpgInitStandard Proc USES EBX hWnd:DWORD

;--------------------------------------------------------------------
; clpgInitStandard - initialize control with default settings
; IN:   control [window] handle
; OUT:  EAX == NULL
;--------------------------------------------------------------------

;-- Get pointer to CpuPgBar struct
	Invoke GetWindowLong,hWnd,0
	mov ebx,eax
    ASSUME ebx:PTR CpuPgBar

;-- set default colors
	mov eax,[ebx].dwCustomInfo   ; check if we have customized forecolors
	test eax,CPLG_INFO_CUSTOMFORECOLORS
	jnz @F
		mov ecx,ColorsWhiteSmoke
		mov [ebx].dwColorBckgr,ecx   ; background color
		mov ecx,ColorsBlack
		mov [ebx].dwColorText,ecx    ; text color	
		mov ecx,ColorsGreen
		mov [ebx].dwColor000,ecx     ; 0% color
		mov ecx,ColorsYellow
		mov [ebx].dwColor050,ecx     ; 50% color
		mov ecx,ColorsRed
		mov [ebx].dwColor100,ecx     ; 100% color
	@@:

;- set initial value
	mov eax,[ebx].dwStyle		 ; check for CpuLoad-Style
	and eax,CLPG_CPULOAD
	jz @F						 ; ignore message for CpuLoad-Style
		mov [ebx].dwValue,-1     ; cpuload initial value
	@@:
	
	
;-- set default fontsize	
	mov eax,[ebx].dwCustomInfo   ; check if we have customized fontsize
	test eax,CPLG_INFO_CUSTOMFONTSIZE
	jnz	@F
		mov [ebx].dwFontSize,70  ; set fontsize to nn% of control height  
	@@:
	
;-- set default timer interval
	mov [ebx].uElapse,500        ; Timer setting (ms)
	
;-- set default range
	mov eax,[ebx].dwCustomInfo   ; check if we have custom range
	test eax,CPLG_INFO_CUSTOMRANGE
	jnz	@F
		mov [ebx].dwMinVal,0         ; Set standard range for control values
		mov [ebx].dwMaxVal,100
		mov [ebx].dwRange,100
	@@:
	
;-- Set default step size
	mov eax,[ebx].dwCustomInfo   ; check if we have custom step	
	test eax,CPLG_INFO_CUSTOMSTEP
	jnz	@F
		mov [ebx].dwStep,10
	@@:
	
    ASSUME ebx:NOTHING
	xor eax,eax
	Ret
ClpgInitStandard EndP

;-- Exported functions

ClpgSetForeColors Proc uses ebx hWnd:DWORD, pColors:DWORD

;-- Get pointer to CpuPgBar struct
	Invoke GetWindowLong,hWnd,0
	mov ebx,eax
    ASSUME ebx:PTR CpuPgBar
;-- Store colors
	mov edx,pColors
	mov ecx,[edx]
	mov [ebx].dwColor000,ecx     ; 0% color
	mov ecx,[edx+4]
    mov [ebx].dwColor050,ecx     ; 50% color
    mov ecx,[edx+8]
	mov [ebx].dwColor100,ecx     ; 100% color
;-- we have custom colors
    or [ebx].dwCustomInfo,CPLG_INFO_CUSTOMFORECOLORS
;-- reinitialize
    Invoke CleanUpGDIp,hWnd
    Invoke InitGDIp,hWnd
	Invoke InitFont,hWnd,NULL
    ASSUME ebx:NOTHING
	xor eax,eax
	Ret
	
ClpgSetForeColors EndP

ClpgGetData PROC uses ebx hWnd:DWORD, ptCLPG_DATA:DWORD


;-- Get pointer to CpuPgBar struct
	Invoke GetWindowLong,hWnd,0
	mov ebx,eax
    ASSUME ebx:PTR CpuPgBar    
    mov edx,ptCLPG_DATA
    ASSUME edx:PTR CPLG_DATA
    
    m2m [edx].dwForeColor000,[ebx].dwColor000
    m2m [edx].dwForeColor050,[ebx].dwColor050
    m2m [edx].dwForeColor100,[ebx].dwColor100
	m2m [edx].dwColorBckgr,[ebx].dwColorBckgr
	m2m [edx].dwColorText,[ebx].dwColorText
	m2m [edx].dwFontSize,[ebx].dwFontSize    

    ASSUME edx:NOTHING
    ASSUME ebx:NOTHING
	xor eax,eax
	Ret
ClpgGetData EndP

ClpgSetData PROC uses ebx hWnd:DWORD, ptCLPG_DATA:DWORD


	Invoke CleanUpGDIp,hWnd

;-- Get pointer to CpuPgBar struct
	Invoke GetWindowLong,hWnd,0
	mov ebx,eax
    ASSUME ebx:PTR CpuPgBar
    mov edx,ptCLPG_DATA
    ASSUME edx:PTR CPLG_DATA

	m2m [ebx].dwColor000,[edx].dwForeColor000
	m2m [ebx].dwColor050,[edx].dwForeColor050
	m2m [ebx].dwColor100,[edx].dwForeColor100
	m2m [ebx].dwColorBckgr,[edx].dwColorBckgr
	m2m [ebx].dwColorText,[edx].dwColorText
	m2m [ebx].dwFontSize,[edx].dwFontSize
    
    ASSUME edx:NOTHING
;-- reinitialize
	or [ebx].dwCustomInfo,CPLG_INFO_CUSTOMFORECOLORS
    ASSUME ebx:NOTHING
    
    Invoke InitGDIp,hWnd
    Invoke InitFont,hWnd,NULL
	Invoke InvalidateRect,hWnd,NULL,TRUE
	xor eax,eax
	Ret
ClpgSetData EndP