COMMENT *-----------------------------------------------------------------------------------------

Logfile Writer  Phoenix 2005

This is just a simple way to create a logfile designed to trace API call results
using conditional assembling. Enabled by LOGMODE EQU TRUE, disabled by LOGMODE EQU FALSE.

FILES:  LogWrite.asm
		Logwrite.inc

IN:		lpszProc: 		address of ansi string, information about proc from where we trace
		lpszText: 		address to ansi string, info about current APi name i. e.
		lpszExpected: 	address to ansi string, info about expected result
		dwRetCode: 		value returned by current Api call (from eax), returned in eax
		dwPos: 			switches header mode (0) to info mode (!=0)
OUT:    eax: 			return value passed in eax so the return value will not be changed

USAGE:	IF LOGMODE
			Invoke WriteLogFile,ADDR szProcName,0,0,0,0
		ENDIF
		
		(dwPos must be ZERO)
		if not existing, a logfile will be created (filename -> szFileName location: LogWrite.inc)
		header information is written (szHeader, szTitle, szLogtime -> location: LogWrite.inc)

		Invoke GlobalAlloc,GPTR,dwBufferSize
		IF LOGMODE
			Invoke WriteLogFile,0,SADD("GlobalAlloc:",13,10),SADD("NONZERO "),eax,1
		ENDIF
		
		(dwPos must be NONZERO)
		appends the following to the logfile:
		
		GlobalAlloc:
    		Expected: NONZERO  Returned: 00137988
    		
NOTE:	The results are always appended to the file, you will have to delete it by hand 
		if you need to start a new one.
    		
-----------------------------------------------------------------------------------------------*
.Code

IF LOGMODE

WriteLogFile PROC lpszProc:DWORD,lpszText:DWORD,lpszExpected:DWORD,dwRetCode:DWORD,dwPos:DWORD

Local lobs:OFSTRUCT
Local hFile:DWORD
Local numBytes:DWORD
Local dwDist:DWORD
Local szPosBuf[64]:BYTE
Local szResult[512]:BYTE

	push dwRetCode

	lea edx,szResult
	xor ecx,ecx
	mov DWORD PTR [edx],ecx
	lea edx,szPosBuf
	mov DWORD PTR [edx],ecx

	Invoke OpenFile,ADDR szFileName,ADDR lobs,OF_EXIST
	.If eax == -1        ; File does NOT exist (HFILE_ERROR == -1)
		Invoke OpenFile,ADDR szFileName,ADDR lobs,OF_CREATE or OF_WRITE
		mov hFile,eax
	.Else                ; File exists 
		Invoke OpenFile,ADDR szFileName,ADDR lobs,OF_WRITE
		mov hFile,eax
	.Endif
	Invoke GetFileSize,hFile,0
	mov dwDist,eax
	invoke SetFilePointer,hFile,dwDist,0,FILE_BEGIN
	
;-- Build String
	.if dwPos == 0
		.if dwDist != 0
			invoke szCatStr, ADDR szResult,ADDR szNL
		.endif
		invoke szCatStr,ADDR szResult,ADDR szHeader
		invoke szCatStr,ADDR szResult,ADDR szTitle
		invoke szCatStr,ADDR szResult,ADDR szHeader
		invoke szCatStr,ADDR szResult,ADDR szNL
		invoke GetOsInfo,ADDR szResult
		invoke szCatStr,ADDR szResult,ADDR szNL
		invoke szCatStr,ADDR szResult,ADDR szLogtime
		Invoke Time2sz, ADDR szResult
		invoke szCatStr,ADDR szResult,SADD(13,10,13,10)
		invoke szCatStr,ADDR szResult,lpszProc
		invoke szCatStr,ADDR szResult,ADDR szNL
	.else
		invoke szCatStr,ADDR szResult,lpszText
		invoke szCatStr,ADDR szResult,ADDR szExpected
		invoke szCatStr,ADDR szResult,lpszExpected
		invoke szCatStr,ADDR szResult,ADDR szReturned
		invoke dw2hex,dwRetCode,ADDR szPosBuf
		invoke szCatStr,ADDR szResult,ADDR szPosBuf
		invoke szCatStr,ADDR szResult,ADDR szNL
	.endif
	
	invoke StrLen,ADDR szResult
	mov edx,eax
	invoke WriteFile,hFile,ADDR szResult,edx,ADDR numBytes,0
	invoke _lclose,hFile

	pop eax
	ret

WriteLogFile EndP

Time2sz PROC lpszResult:DWORD

Local sysT:SYSTEMTIME
Local dwMon:DWORD
Local dwDay:DWORD
Local dwYear:DWORD
Local dwHour:DWORD
Local dwMin:DWORD
Local dwSec:DWORD
Local szTmpBuf[32]:BYTE

	invoke GetLocalTime,ADDR sysT
;-- convert WORD values to DWORD values
    ; There must be a better way....
	xor ecx,ecx
	mov cx,sysT.wMonth
	mov dwMon,ecx
	mov cx,sysT.wDay
	mov dwDay,ecx
	mov cx,sysT.wYear
	mov dwYear,ecx
	mov cx,sysT.wHour
	mov dwHour,ecx
	mov cx,sysT.wMinute
	mov dwMin,ecx
	mov cx,sysT.wSecond
	mov dwSec,ecx
		
	invoke wsprintf,ADDR szTmpBuf,ADDR szTime,dwMon,dwDay,dwYear,dwHour,dwMin,dwSec
	invoke szCatStr,lpszResult,ADDR szTmpBuf
	
	xor eax,eax
	ret
		
Time2sz ENDP

GetOsInfo Proc lpszResult:DWORD

Local hKey:DWORD
Local cbData:DWORD
Local dwType:DWORD
Local osEX:OSVINFOEX
Local szVersion[MAX_PATH]:BYTE
Local szVerBuf[MAX_PATH]:BYTE

	mov	osEX.dwOSVersionInfoSize,sizeof(OSVINFOEX)
	invoke GetVersionEx,ADDR osEX 
	.if eax == 0
		JMP _error
	.endif

	mov cbData,MAX_PATH
	mov dwType,REG_SZ

; -- Win32s
	.if osEX.dwPlatformId == VER_PLATFORM_WIN32s				; 0
		invoke lstrcat,ADDR szVerBuf,ADDR szWin32s				; Win32s
; -- Win95 family
	.elseif osEX.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS 	; 1
		; Get the correct OS name from Registry
		Invoke RegOpenKeyEx,HKEY_LOCAL_MACHINE,ADDR szKey,0,KEY_READ,ADDR hKey
		Invoke RegQueryValueEx,hKey,ADDR szProdName,NULL,ADDR dwType,ADDR szVerBuf,ADDR cbData 
		Invoke RegCloseKey,hKey
; -- WinNT family
	.elseif osEX.dwPlatformId == VER_PLATFORM_WIN32_NT			; 2
		Invoke RegOpenKeyEx,HKEY_LOCAL_MACHINE,ADDR szKeyNT,0,KEY_READ,ADDR hKey
		Invoke RegQueryValueEx,hKey,ADDR szProdName,NULL,ADDR dwType,ADDR szVerBuf,ADDR cbData 
		Invoke RegCloseKey,hKey
	.endif
	
	mov ecx,osEX.dwBuildNumber
	and ecx,0ffffh
	.if osEX.dwPlatformId > 0
		invoke wsprintf,ADDR szVersion, ADDR szBuild,osEX.dwMajorVersion,osEX.dwMinorVersion,ecx
		invoke lstrcat,ADDR szVerBuf,ADDR szVersion
		.if osEX.dwPlatformId == VER_PLATFORM_WIN32_NT
			invoke lstrcat,ADDR szVerBuf,ADDR osEX.szCSDVersion
		.endif
	.endif

	invoke lstrcat,lpszResult,ADDR szVerBuf

	xor eax,eax
	RET
_error:
	mov eax,TRUE
	RET
GetOsInfo EndP

ENDIF
